<?php

/* load translations */
load_theme_textdomain('royalgold', get_template_directory() . '/languages');

$reinstall_error_msg = __( 'Please reinstall the theme, one of the main files is missing.', 'royalgold' );

/* add admin options */
define( 'ADMIN_PATH', get_template_directory() . '/utils/admin/' );
define( 'ADMIN_DIR', get_template_directory_uri() . '/utils/admin/' );
if (file_exists( get_template_directory() . '/utils/admin/index.php' )) {
	require_once( 'utils/admin/index.php' );
} else {
	wp_die( $reinstall_error_msg );
}

/* meta box */
if ( ! class_exists( 'RW_Meta_Box' ) && ! defined('RWMB_URL') ) {
	define( 'RWMB_URL', get_template_directory_uri() . '/utils/meta-box/' );
	define( 'RWMB_DIR', get_template_directory() . '/utils/meta-box/' );
	if (file_exists( RWMB_DIR . 'meta-box.php' ))
		require_once RWMB_DIR . 'meta-box.php';
	else
		wp_die( $reinstall_error_msg );
}
if (file_exists( get_template_directory() . '/utils/meta-box/meta-box-config.php' ))
	include get_template_directory() . '/utils/meta-box/meta-box-config.php';
else
	wp_die( $reinstall_error_msg );

/* add image resizer script */
if (file_exists( get_template_directory() . '/utils/aq_resizer.php' )) {
	require_once( 'utils/aq_resizer.php' );
} else {
	wp_die( $reinstall_error_msg );
}

/* basic shortcodes */
if (file_exists(get_template_directory() . '/utils/shortcodes.php')) {
	require_once('utils/shortcodes.php');
} else {
	wp_die( $reinstall_error_msg );
}

/* Register the required plugins for this theme with TGM Plugin Activation class */
if (is_admin() && file_exists(get_template_directory() . '/utils/shortcodes.php')) {
	
	require_once('utils/tgm/class-tgm-plugin-activation.php');

	add_action( 'tgmpa_register', 'royalgold_register_required_plugins' );
	function royalgold_register_required_plugins() {
		$plugins = array(
			array(
				'name' => '360&deg; Panoramic Viewer ',
				'slug' => 'panorama360',
				'source' => get_template_directory() . '/utils/tgm/plugins/panorama360.zip',
				'required' => false,
				'version' => '1.1',
				'force_activation' => false,
				'force_deactivation' => false,
				'external_url' => '',
			),
			array(
				'name' => 'easyReservations',
				'slug' => 'easyreservations',
				'required' => false,
			),
			array(
				'name' => 'Responsive Lightbox',
				'slug' => 'responsive-lightbox',
				'required' => false,
			),
			array(
				'name' => 'Contact Form 7',
				'slug' => 'contact-form-7',
				'required' => false,
			),
		);

		tgmpa( $plugins );
	}
}

/* hide inline comments style - it is already added in the theme style file */
function remove_recent_comments_style() {
	global $wp_widget_factory;
	remove_action( 'wp_head', array( $wp_widget_factory->widgets['WP_Widget_Recent_Comments'], 'recent_comments_style' ) );
}
add_action( 'widgets_init', 'remove_recent_comments_style' );
add_theme_support( 'automatic-feed-links' );
add_editor_style();
if ( ! isset( $content_width ) ) $content_width = 480;

/* register menu */
register_nav_menus( array(
	'main_menu' => __( 'Main Menu', 'royalgold' )
) );

/* enable post thumbnails */
add_theme_support( 'post-thumbnails' );

/* pagination utils */
function paginate_links_attributes() {
	return 'class="button"';
}
add_filter('paginate_links_attributes', 'paginate_links_attributes');
function pagination_links() {
	global $wp_query;
	$max = $wp_query->max_num_pages;
	if ($max > 1) {
		if (!$current = get_query_var('paged')) $current = 1;
		$a['base'] = str_replace(99999999, '%#%', get_pagenum_link(99999999));
		$a['total'] = $max;
		$a['current'] = $current;
		$a['mid_size'] = 1;
		echo '<div class="pagination">'.paginate_links($a).'</div>';
	}
}

/* excerpt settings */
function royalgold_excerpt_more($more) {
	global $post;
	return '&#46;&#46;&#46; <a href="'. get_permalink($post->ID) . '" class="more">' . __( '(more)', 'royalgold' ) . '</a>';
}
add_filter('excerpt_more', 'royalgold_excerpt_more');
function royalgold_excerpt_length( $length ) {
	return 42;
}
add_filter('excerpt_length', 'royalgold_excerpt_length', 999);

/* sidebar widgets */
function royalgold_widgets_init() {
	register_sidebar( array(
		'name' => __( 'Content Footer Widgets', 'royalgold' ),
		'id' => 'page-sidebar',
		'description' => __( 'Widget area for your main site sidebar displayed at the bottom of each page.', 'royalgold' ),
		'before_widget' => '<div class="sep"><span></span></div><div class="%2$s page-widget">',
		'after_widget' => '</div>',
		'before_title' => '<h3>',
		'after_title' => '</h3>',
	));
}
add_action( 'widgets_init', 'royalgold_widgets_init' );

// functions to run on activation - flush to clear rewrites
if ( is_admin() ) {
	global $pagenow;
	if ( isset($_GET['activated'] ) && $pagenow == 'themes.php' ) {
		$wp_rewrite->flush_rules();
	}
	// check for theme updates
	if ( !empty($smof_data["themeforest_username"]) && !empty($smof_data["themeforest_apikey"]) ) {
		require_once("utils/updater/theme-update.php");
		ThemeUpdater::init( $smof_data["themeforest_username"], $smof_data["themeforest_apikey"], 'liviu_cerchez' );
	}
}

/* remove orphan <p> and <br /> elements - 100% safe doesn't remove any WordPress functionality -  */
function content() {
	return str_replace(']]>', ']]&gt;', apply_filters('the_content', do_shortcode(get_the_content())));
}

// enqueue the scripts
function royalgold_scripts_and_style_function() {

	global $smof_data;

	// enqueue google fonts
	if (!empty($smof_data['font_body']) || !empty($smof_data['font_headers'])) {
		// enqueue google fonts
		$fonts  = (!empty($smof_data['font_body'])) ? ($smof_data['font_body'] . '|') : '';
		$fonts .= (!empty($smof_data['font_headers'])) ? $smof_data['font_headers'] : '';
		$fonts  = trim( $fonts, '|' );
		$protocol = is_ssl() ? 'https' : 'http';
		$font_subset = (!empty($smof_data['google_fonts_subset'])) ? '&subset=' . $smof_data['google_fonts_subset'] : '';
		wp_register_style( 'googlefonts', $protocol . '://fonts.googleapis.com/css?family='. rawurlencode($fonts) . $font_subset );
		wp_enqueue_style( 'googlefonts' );
	}

	wp_register_style( 'main', get_stylesheet_directory_uri() . '/style.css' );
	wp_enqueue_style( 'main' );

	$params = '?color_base=' . rawurlencode($smof_data['color_base']);
	if (!empty($smof_data['color_link'])) $params .= '&amp;color_link=' . rawurlencode($smof_data['color_link']);
	if (!empty($smof_data['color_background'])) $params .= '&amp;color_bg=' . rawurlencode($smof_data['color_background']);
	if (!empty($smof_data['bg_pattern'])) $params .= '&amp;pattern=' . rawurlencode(basename($smof_data['bg_pattern']));
	if (!empty($smof_data['extra_space_menu'])) $params .= '&amp;extra_space_menu=' . rawurlencode($smof_data['extra_space_menu']);
	if (!empty($smof_data['extra_space_content'])) $params .= '&amp;extra_space_content=' . rawurlencode($smof_data['extra_space_content']);
	if (!empty($smof_data['menu_text_case'])) $params .= '&amp;menu_case=' . rawurlencode($smof_data['menu_text_case']);
	if (!empty($smof_data['font_body'])) $params .= '&amp;font_body=' . rawurlencode($smof_data['font_body']);
	if (!empty($smof_data['font_headers'])) $params .= '&amp;font_headers=' . rawurlencode($smof_data['font_headers']);
	wp_register_style( 'custom-override', get_template_directory_uri() . '/style-custom.php' . $params );
	wp_enqueue_style( 'custom-override' );

	wp_register_style( 'easy-form-royalgold', get_template_directory_uri() . '/style-easy-form.css' );

	// jquery - required in header
	wp_enqueue_script('jquery');
	if ( is_singular() ) wp_enqueue_script( 'comment-reply' );

	global $is_IE;
	if( $is_IE ) {
		wp_enqueue_script( 'ie_js', get_template_directory_uri() . '/js/ie.js' );
	}

	wp_enqueue_script('site_js', get_template_directory_uri() . '/js/site.js', array('jquery'), false, true);
	if (!empty($smof_data['custom_scrollbars'])) {
		wp_enqueue_script('nicescroll-js', get_template_directory_uri() . '/js/nicescroll.min.js', array('site_js'), false, true);
	}
	wp_enqueue_script('supersized_js', get_template_directory_uri() . '/js/supersized.min.js', array('site_js'), false, true);
}
add_action( 'wp_enqueue_scripts', 'royalgold_scripts_and_style_function' );

/* comments section */
if ( ! function_exists( 'royalgold_comment' ) ) {

	function royalgold_comment( $comment, $args, $depth ) {
		$GLOBALS['comment'] = $comment;
		switch ( $comment->comment_type ) :
			case 'pingback' :
			case 'trackback' :
			// Display trackbacks differently than normal comments.
		?>
			<li <?php comment_class(); ?> id="comment-<?php comment_ID(); ?>">
					<p><?php _e( 'Pingback:', 'royalgold' ); ?> <?php comment_author_link(); ?> <?php edit_comment_link( __( '(Edit)', 'royalgold' ) ); ?></p>
<?php
				break;
			default :
			// Proceed with normal comments.
			global $post; ?>

			<li <?php comment_class(); ?> id="comment-<?php comment_ID(); ?>">
				<div class="clearfix">
					<div class="comment-author"><?php echo get_avatar( $comment, 80 ); ?></div>
					<div class="comment-body">
						<h6><?php echo get_comment_author_link( $comment->comment_ID ); ?> <?php comment_reply_link( array_merge( $args, array( 'reply_text' => __( '(reply)', 'royalgold' ), 'depth' => $depth, 'max_depth' => $args['max_depth'] ) ) ); ?></h6>
						<p class="date"><a href="<?php echo esc_url( get_comment_link( $comment->comment_ID ) ); ?>"><?php /* 1: date, 2: time */
							printf( __( '%1$s at %2$s', 'royalgold' ), get_comment_date(), get_comment_time() ); ?></a></p>
<?php if ( $comment->comment_approved == '0' ) : ?>
						<div class="alert notice">
							<p><em><?php _e( 'Your comment is awaiting moderation.', 'royalgold' ); ?> <?php edit_comment_link( __( 'Edit', 'royalgold' ) ); ?></em></p>
							<br />
							<?php comment_text(); ?>
						</div>
<?php else: ?>
						<?php comment_text(); ?>
						<?php edit_comment_link( __( 'Edit', 'royalgold' ) ); ?>
<?php endif; ?>

					</div>
				</div>
<?php
			break;
		endswitch; // end comment_type check
	}

} // function_exists